% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convaq.R, R/zzz.R
\docType{package}
\name{convaq}
\alias{convaq}
\alias{convaq}
\alias{convaq-package}
\title{Perform CNV-based association study.}
\usage{
convaq(segments1, segments2, model, name1 = "Group 1", name2 = "Group 2",
  qvalues = FALSE, qvalues.rep = 4000, merge = FALSE,
  merge.threshold = 0, p.cutoff = 0.05, pred1 = NULL, pred2 = NULL,
  nthreads = NULL)
}
\arguments{
\item{segments1}{Data frame of segments for group 1. See details.}

\item{segments2}{Data frame of segments for group 2. See details.}

\item{model}{Model type. Either "statistical" or "query".}

\item{name1}{Name of first group.}

\item{name2}{Name of second group.}

\item{qvalues}{TRUE if q-values should be computed, FALSE otherwise.}

\item{qvalues.rep}{Number of repetitions to use in q-value computation.}

\item{merge}{TRUE if adjacent regions of same type should be merged.}

\item{merge.threshold}{Maximum number of base pairs allowed between two regions in order to be adjacent.}

\item{p.cutoff}{(statistical model) P-value cutoff in statistical model.}

\item{pred1}{(query model) Predicate for group 1 in query model.}

\item{pred2}{(query model) Predicate for group 2 in query model.}

\item{nthreads}{Number of threads to use. Defaults to number of cores available.}
}
\value{
An object of class \code{convaq} with the following elements:
  \item{regions}{Data frame of significant regions.}
  \item{freq}{Data frame of within-group variation frequencies for each reported region.}
  \item{state}{The states of individual patients/samples for each region.}
  \item{model}{Type of model used.}
  \item{name1}{Name of first group.}
  \item{name2}{Name of second group.}
  \item{qvalues}{True if q-values were computed.}
  \item{qvalues.rep}{Number of repetitions used in q-value computation.}
  \item{merge}{True if adjacent regions of same type should be merged.}
  \item{merge.threshold}{Maximum distance (in base pairs) allowed between merged regions.}
  \item{p.cutoff}{P-value cutoff (statistical model only).}
  \item{pred1}{Predicate for group 1 (query model only).}
  \item{pred2}{Predicate for group 2 (query model only).}
}
\description{
CoNVaQ is a method for performing CNV-based association studies. It provides two models:
a query-based model and a standard statistical model using Fisher's exact test.

convaq
}
\section{Segment data format}{

The CNV segment sets \code{segments1} and \code{segments2} must be data frame objects with the following five columns:
\describe{
  \item{patient}{Patient identifier for the patient/sample the segment was found in.}
  \item{chr}{Chromosome the segment is located in.}
  \item{start}{First position of the segment in base pairs.}
  \item{end}{Last position of the segment in base pairs.}
  \item{type}{Segment type. One of "Gain", "Loss" or "LOH".}
}
The order of the columns is used in order to determine the contents, not the column names.
}

\section{Predicate format}{

Predicates are given as a character vector with the following format:
\preformatted{"[COMP] [FREQ] [EQ] [TYPE]"}
where:
\describe{
  \item{COMP}{is a comparison operator. One of "<" (less than), ">" (greater than), "<=" (less than or equal to) or ">=" (greater than or equal to).}
  \item{FREQ}{is a numerical value between 0 and 1.}
  \item{EQ}{is either "==" (equal to) or "!=" (not equal to).}
  \item{TYPE}{is a segment type. One of "Gain", "Loss", "LOH" or "Normal".}
}
The four arguments must be separated by a single space.

A predicate is evaluated for a specific group in a each region, and will either evaluate to TRUE or FALSE.
If we want to find regions where at least 50\% of the patients in a group are reported as "Gain", we can use the predicate:
\preformatted{">= 0.5 == Gain"}
Likewise if we are searching for regions where less than 25\% of the patients in a group have any kind of variation,
we can use the predicate
\preformatted{"< 0.25 != Normal"}
The query model works by combining two predicates, one for each group of segments.
For instance, we can combine two predicates to search for regions where at least 50\% of patients in the first group have a "Gain",
and less than 25\% of patients in the second group:
\preformatted{convaq(s1, s2, model="query", pred1=">= 0.5 == Gain", pred2="< 0.25 == Gain")}
}

\examples{
data("example", package="convaq")
s1 <- example$disease
s2 <- example$healthy

# statistical model
convaq(s1, s2, model="statistical", p.cutoff=0.05, qvalues=FALSE)
convaq(s1, s2, model="statistical", p.cutoff=0.05, qvalues=TRUE, qvalues.rep=2000)

# query model
convaq(s1, s2, model="query", pred1=">= 0.5 == Gain", pred2="<= 0.2 == Gain")
convaq(s1, s2, model="query", pred1=">= 0.6 != Normal", pred2=">= 0.6 == Normal")

}
\author{
Simon J. Larsen <simonhffh@gmail.com>
}
